import axios, {AxiosError, AxiosInstance, AxiosRequestConfig, AxiosResponse, Method} from "axios";
import {Store} from "vuex";
import {VueRouter} from "vue-router/types/router";
import {ServerRetEnum} from "@/views/admin/types/Types";

const sprintf = require('sprintf-js')

let apiHost = process.env.NODE_ENV !== 'production' ? process.env.VUE_APP_API_HOST : document.location.host;
let scheme = document.location.protocol;


class Http {
    private readonly _handler : AxiosInstance;
    static _instance : Http;
    private _store? : Store<any>;
    private _router : VueRouter | any;
    private _vue : Vue | any;
    private _baseUri : string = '';

    static instance() : Http {
        if (!this._instance) {
            this._instance = new Http();
        }
        return this._instance;
    }

    set store(store : Store<any>) {
        this._store = store;
    }

    set router(router : VueRouter) {
        this._router = router;
    }

    set vue(vue : Vue) {
        this._vue = vue;
    }

    get handler () : AxiosInstance {
        return this._handler;
    }

    get uri() {
        // console.log(this._handler.defaults, 'get_uri');
        return this._handler.defaults.baseURL;
    }

    set baseUri(uri : string) {
        this._baseUri = uri;
    }

    get baseUri () : string {
        return this._baseUri;
    }

    constructor() {
        this._handler = axios.create({
            baseURL : scheme + "//" + apiHost,
            timeout : 50000,
        });
        this._handler.interceptors.response.use((response : AxiosResponse) => {
            // console.log(response, '基本请求结果')
            if (response.status == 200) {
                return response.data;
            } else {
                // console.log(response, '未知错误')
            }
        }, (error : AxiosError) => {
            let response = error.response;
            if (response?.status === 401) {
                this._router.replace("/login");
            } else {
                console.error('response error', error.response);
            }
            return Promise.reject(error);
        })
        this._handler.interceptors.request.use((config : AxiosRequestConfig) => {
            config.headers["X-Requested-With"] = "XMLHttpRequest";
            if (this._store) {
                config.headers["Authorization"] = "Bearer " + this._store.getters.token;
            }
            return config;
        })
    }

    public sprintf(...params) : string {
        return sprintf.sprintf(...params);
    }

    public get(url : string, params? : object | any, ...args : any[]) : Promise<ResponseInterface> {
        return this.request('GET', url, params, ...args);
    }

    public post(url : string, data? : object | any, ...args : any[]) : Promise<ResponseInterface> {
        return this.request('POST', url, data, ...args);
    }

    public put(url : string, params? : object | any, ...args : any[]) : Promise<ResponseInterface> {
        return this.request('PUT', url, params, ...args);
    }

    public delete(url : string, params? : object | any, ...args : any[]) : Promise<ResponseInterface> {
        return this.request('DELETE', url, params, ...args);
    }

    public request(method : Method, url : string, data? : object | any, headers? : any) : Promise<ResponseInterface> {
        url = this._baseUri + url;
        let params : AxiosRequestConfig = {
            url, method, headers
        }
        if (method.toLocaleUpperCase() === 'GET') {
            params.params = data;
        } else {
            params.data = data;
        }
        return new Promise<ResponseInterface>((resolve, reject) => {
            this._handler.request(params).then((res : any) => {
                if (res.code === ServerRetEnum.SUCCESS) {
                    resolve(res);
                } else if(res.code === ServerRetEnum.NO_PERMISSION) {
                    if(this._vue.$Message){
                        this._vue.$Message.error(res.message);
                    }
                } else {
                    console.error(res);
                    reject(res.message);
                }
            }).catch((err : any) => {
                reject(err);
            })
        })
    }

    public download(method : Method, url: string) : Promise<AxiosResponse> {
        this._handler.interceptors.response.use((response : AxiosResponse) => {
            return Promise.resolve(response)
        });
        return new Promise((resolve, reject) => {
            url = this._baseUri + url;
            let params : AxiosRequestConfig = {
                url, method, responseType: "blob", timeout: 600e3
            }
            this._handler.request(params).then((res : any) => {
                resolve(res);
            }).catch((reason) => {
                console.error(reason);
                reject(reason);
            })
        });
    }
}

const instance : Http = Http.instance();
export default instance;
export const get = instance.get;
export const post = instance.post;


export interface ResponseInterface {
    data? : Array<any> | object | object[] | any,
    code : ServerRetEnum,
    message : string
}

export interface ResponseErrorInterface {
    result : number,
    message : string
}

export interface RequestInterface {
    store : Store<any>;
    router : VueRouter;
    handler : AxiosInstance;
    baseUri : string;

    instance();


    get(url : string, params? : object | any, ...args : any[]) : Promise<ResponseInterface>;

    post(url : string, params? : object | any, ...args : any[]) : Promise<ResponseInterface>;

    put(url : string, params? : object | any, ...args : any[]) : Promise<ResponseInterface>;

    delete(url : string, params? : object | any, ...args : any[]) : Promise<ResponseInterface>;

    request(method : Method, url : string, data? : object | any, headers? : any) : Promise<ResponseInterface>;

    download(method : Method, url : string) : Promise<AxiosResponse>
}